# This file is part of GNU Health.  The COPYRIGHT file at the top level of
# this repository contains the full copyright notices and license terms.
from .many2one import Many2One


class One2One(Many2One):
    pass
